package com.samelement.iot_notification.appwidget;

import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Typeface;
import android.view.View;
import android.widget.RemoteViews;

import androidx.core.graphics.ColorUtils;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.Legend;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.github.mikephil.charting.formatter.ValueFormatter;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;
import com.github.mikephil.charting.utils.ColorTemplate;
import com.samelement.iot_notification.R;
import com.samelement.iot_notification.appwidget.db.AppWidgetObject;
import com.samelement.iot_notification.appwidget.db.WidgetReaderDbHelper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

public class DemoAppWidget extends AppWidgetProvider {

    private static final int[] colors = new int[]{
            ColorTemplate.MATERIAL_COLORS[0],
            ColorTemplate.MATERIAL_COLORS[1],
            ColorTemplate.MATERIAL_COLORS[2],
            ColorTemplate.MATERIAL_COLORS[3]
    };
    private static final int MAX_ITEMS = 5;

    private static LineDataSet initLineDataSet(Context context, ArrayList<Entry> values, String label, int color) {
        LineDataSet d = new LineDataSet(values, label);
        d.setLineWidth(8f);
        d.setCircleRadius(10f);
        d.setValueTextSize(25f);
        d.setValueTypeface(Typeface.DEFAULT_BOLD);
        d.setValueTextColor(context.getResources().getColor(R.color.colorGrey));

        ValueFormatter valueFormatter = new ValueFormatter() {
            @Override
            public String getFormattedValue(float value) {
                return round(value, 2);
            }
        };
        d.setValueFormatter(valueFormatter);

        d.setColor(color);
        d.setCircleColor(color);

        return d;
    }

    private static String round(float value, int places) {
        if (places < 0) return String.valueOf(value);

        BigDecimal bd = new BigDecimal(value);
        bd = bd.setScale(places, RoundingMode.HALF_UP);
        return bd.toString();
    }

    private static LineChart initLineChart(Context context, boolean isDark) {
        LineChart chart = new LineChart(context);
        chart.setDrawGridBackground(false);
        chart.getDescription().setEnabled(false);
        chart.setDrawBorders(false);
        chart.setExtraOffsets(50f, 50f, 50f, 50f);

        if (isDark) {
            chart.setBackgroundColor(context.getResources().getColor(R.color.colorBlackLightGrey));
        }
        else chart.setBackgroundColor(context.getResources().getColor(R.color.colorContentLight));

        chart.getAxisLeft().setEnabled(false);
        chart.getAxisRight().setDrawAxisLine(false);
        chart.getAxisRight().setDrawGridLines(false);
        chart.getXAxis().setDrawAxisLine(false);
        chart.getXAxis().setDrawGridLines(false);
        chart.getXAxis().setDrawLabels(false);
        chart.getAxisRight().setDrawLabels(false);

        Legend l = chart.getLegend();
        l.setVerticalAlignment(Legend.LegendVerticalAlignment.BOTTOM);
        l.setHorizontalAlignment(Legend.LegendHorizontalAlignment.LEFT);
        l.setOrientation(Legend.LegendOrientation.HORIZONTAL);
        l.setTextSize(25);
        l.setFormSize(16);
        l.setDrawInside(false);
        l.setTypeface(Typeface.DEFAULT_BOLD);
        l.setHorizontalAlignment(Legend.LegendHorizontalAlignment.CENTER);
        l.setXEntrySpace(16);
        l.setTextColor(context.getResources().getColor(R.color.colorGrey));

        return chart;
    }

    private static ArrayList<Entry> initEntries(String value) {
        ArrayList<Entry> series = new ArrayList<>();
        // loop values
        if (value.contains(";")) {
            // go split
            String[] splitted = value.split(";");
            for (int i = 0; i < splitted.length; i++) {
                if (i + 1 >= splitted.length) {
                    series.add(new Entry(MAX_ITEMS - 1, Float.parseFloat(splitted[i])));
                } else {
                    series.add(new Entry(i, Float.parseFloat(splitted[i])));
                }
            }
        } else {
            series.add(new Entry(MAX_ITEMS - 1, Float.parseFloat(value)));
        }


        return series;
    }

    static void updateAppWidget(Context context, AppWidgetManager appWidgetManager,
                                int appWidgetId) {
        WidgetReaderDbHelper dbHelper = WidgetReaderDbHelper.getInstance(context);
        List<AppWidgetObject> appWidgetObjects = dbHelper.getById(appWidgetId + "");

        RemoteViews rv = new RemoteViews(context.getPackageName(), R.layout.demo_app_widget);

        boolean isDark = true;
        if (appWidgetObjects.size() > 0) {
            AppWidgetObject firstObj = appWidgetObjects.get(0);
            String title = firstObj.getTitle();
            rv.setTextViewText(R.id.appwidget_title, title);

            isDark = firstObj.getColor().equals("dark");

            if (isDark) {
                rv.setInt(R.id.rootLayout, "setBackgroundColor", context.getResources().getColor(R.color.colorContentDark));
            } else {
                rv.setInt(R.id.rootLayout, "setBackgroundColor", context.getResources().getColor(R.color.colorContentLight));
            }
        }

        LineChart chart = initLineChart(context, isDark);

        ArrayList<ILineDataSet> dataSets = new ArrayList<>();

        for (int i = 0; i < appWidgetObjects.size(); i++) {
            AppWidgetObject obj = appWidgetObjects.get(i);
            String resourceName = obj.getResource();
            String unit = obj.getUnit();
            if (unit != null && unit.length() > 0 && !unit.equals("null")) {
                resourceName = resourceName + " / " + unit;
            }
            dataSets.add(initLineDataSet(context, initEntries(obj.getValue()), resourceName, colors[i]));
        }

        LineData data = new LineData(dataSets);
        chart.setData(data);

        chart.measure(View.MeasureSpec.makeMeasureSpec(1200, View.MeasureSpec.EXACTLY),
                View.MeasureSpec.makeMeasureSpec(1200, View.MeasureSpec.EXACTLY));
        chart.layout(0, 0, chart.getMeasuredWidth(), chart.getMeasuredHeight());

        Bitmap chartBitmap = chart.getChartBitmap();
        chart.invalidate();


        rv.setBitmap(R.id.imgView, "setImageBitmap", chartBitmap);
        appWidgetManager.updateAppWidget(appWidgetId, rv);
    }


    @Override
    public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
        // There may be multiple widgets active, so update all of them
        for (int appWidgetId : appWidgetIds) {
            updateAppWidget(context, appWidgetManager, appWidgetId);
        }
    }
}
